import os
from quart import Quart, jsonify, request, render_template
from quart_cors import cors
from datetime import datetime
from configs import config as app_config
from threading import Thread
import asyncio
from pynovatec import helpers
# from controllers import controllers
import machineqc
import sensor_calibrate
import livesensor_data_stream
import railcar_data_stream
from luciauth import auth_bp
from module_loader import load_modules


app = Quart(__name__, static_url_path="/")
app = cors(app, allow_origin="*")
app.config['MAX_CONTENT_LENGTH'] = 1024 * 1024 * 50


loop = asyncio.new_event_loop()
app.async_event_loop = loop
asyncio.set_event_loop(loop)

#from modules.baseline import baseline
# import modules.baseline.baseline as baseline

# from modules import baseline


load_modules(app)

# baseline.app_init(app)
#app.register_blueprint(baseline.baseline_bp)

sensor_calibrate.set_asyncio_loop(loop)
machineqc.set_asyncio_loop(loop)
#baseline.set_asyncio_loop(loop)

# livesensor_data_stream.set_asyncio_loop(loop)
# railcar_data_stream.set_asyncio_loop(loop)

app.register_blueprint(livesensor_data_stream.ls_realtime_data_bp)
app.register_blueprint(railcar_data_stream.rc_realtime_data_bp)
app.register_blueprint(sensor_calibrate.calibration_bp)
app.register_blueprint(machineqc.machineqc_bp)
# app.register_blueprint(controllers, url_prefix="/api")
app.register_blueprint(auth_bp)


if __name__ == '__main__':
    #app.run(host="0.0.0.0", debug=True)
    app.run(host="0.0.0.0", port=app_config["server"]["port"],debug=True, loop=loop)
