from quart.blueprints import Blueprint
from configs import config as app_config
import json
from ZmqEventPoster import ZmqEventPoster,ZmqDataEvents
from pynovatec.events import EventListener, EventRegisterListener
from ZmqPostController import user_count_inc, user_count_dec
from quart import websocket
from asyncio import Queue
import asyncio
from queue import Queue as pyQueue

ls_realtime_data_bp = Blueprint('ls_realtime_data_controller', __name__)
connected_realtime_data_websockets = list()
message_queue = []

def push_message_out(msg):
    global message_queue

    for queue in message_queue:
        queue.put(msg)

class ZMQData():
    def __init__(self):        
        self.data_type_names = {
            "4": "Basic",
            "6": "Advanced",
            "10": "Motor failure data",
            "20": "Energy data"
        }

    def process_data(self, val):
        try:
            data = val['data']
            if "CDA_" in data['dataType']:
                data['sensorType']='CDA'
                data['index'] = data['dataTypeIndex']
                data['sensor_id'] = data['uuid']+"_CDA_"+data['dataTypeIndex']

            data['dataTypeName'] = self.data_type_names.get(data['dataType'],data['dataType'])  # Name defaults to the
            data['index'] = int(data['index'])

            send_data = json.dumps({
                'type':'sensorData',
                'data': data
            })
            push_message_out(send_data)

        except Exception as e:
             print("realtime_data_zmq exception: ",str(e))

async def LiveSensorWebsocketBroadcast(ws,queue):
    for message in iter(queue.get,'STOP'):
        # print("Have mesage to send: ", message)
        # print("Qsize: ",queue.qsize())
        await ws.send(message)

zmq_data = ZMQData()

class ZmqListener(EventListener):        
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)      
        self.events.append(ZmqDataEvents)  
                                              
    def handle(self,msg):               
        #print("ZMQ Listener run: ",msg.data)
        zmq_data.process_data(msg.data)
        pass

EventRegisterListener(ZmqListener())

@ls_realtime_data_bp.websocket('/socket.io/')
async def realtime_data_ws():
    global connected_realtime_data_websockets
    global message_queue

    try:
        queue = Queue()
        mq = pyQueue()
        connected_realtime_data_websockets.append(queue)
        message_queue.append(mq)
        try:
            task = asyncio.ensure_future(LiveSensorWebsocketBroadcast(websocket._get_current_object(),mq))
            user_count_inc()
            # x = await connect_websocket(queue)
            await task
            # return x
        finally:
            # websocket disconnected remove the queue from list
            print('live sensor websocket disconnected')
            user_count_dec()
            mq.put('STOP')
            task.cancel()
            connected_realtime_data_websockets.remove(queue)
            message_queue.remove(mq)
            # await task
            
    except Exception as e:
        # print("collect_realtime_data_websocket: ",str(e))
        # close connection, user is not authorized
        await websocket.close(1000, 'UNAUTHORIZED')
        user_count_dec()

async def connect_websocket(queue):
    """Called when a new websocket connects"""
    # print('realtime_data_ws: connected')
    while True:
        data = await queue.get()
        try:
            await websocket.send(data)
        except Exception as e:
            pass
