from quart.blueprints import Blueprint
from configs import config as app_config
import ubus
from uci import Uci
import random
from quart.helpers import make_response
from quart import redirect, request, redirect
import requests
import os
from os import getenv as env
import json

auth_bp = Blueprint('luciauth_controller', __name__)

def createSession():
	u = Uci()
	timeout = u.get('luci', 'sauth', 'sessiontime')
	login = ubus.call("session", "login", {
		"username": "admin",
		"password": "password",
		"timeout": int(timeout) if timeout else None
	})
	#TODO Need to account for failed ubus call
	return login[0]['ubus_rpc_session']

def randomid(num_bytes):
	bytes = []
	while num_bytes > 0:
		bytes.append(format(random.randint(0,255), '02x'))
		num_bytes -= 1
	return ''.join(bytes)

def setSession(sid):
	ubus.call("session", "set", {
		"ubus_rpc_session": sid,
		"values": { "token": randomid(16) }
	})

@auth_bp.route('/luciauth-login', methods=['GET'])
async def login():
	token = request.args.get('token')
	mdh_path = request.args.get('mdh_path')

	# validate_url = request.args.get('validate_url')
	mdh_backend_protocol = env('MDH_API_PROTOCOL', "http://")
	mdh_backend_host = env('MDH_API_HOST', "mdh.pumpsense")
	system_protocol = env('SYSTEM_PROTOCOL', 'http://')
	#system_host = env('SYSTEM_HOST', '192.168.50.3')
	system_host = request.host
	redirect_url = f"{system_protocol}{system_host}/luciauth"
	validate_url = f"{mdh_backend_protocol}{mdh_backend_host}/auth/validate?token={token}"

	if token is None:
		request_token_url = f"{mdh_path}/auth/oauth?redirect_url={redirect_url}"
		# encoded_url = urllib.parse.quote(request_token_url)
		return redirect(request_token_url)
	
	response = requests.request("GET", validate_url)
	if (response.status_code!= 200):
		return redirect('/cgi-bin/luci/')
	
	data = json.loads(response.text)['data']
	if "email" not in data:
		return redirect('/cgi-bin/luci/')

	try:
		ubus.connect("/var/run/ubus/ubus.sock")
	except:
		pass
	session = createSession()
	setSession(session)
	try: 
		ubus.disconnect()
	except:
		pass	

	res = await make_response(redirect('/cgi-bin/luci/admin/status/overview'))
	res.set_cookie('sysauth_http',session, None, None, '/cgi-bin/luci/', None, False, True, "lax")
	return res