import sys
sys.path.append('/opt/novatec/machine-qc')

import time
import base64
from asyncio import Queue
import urllib.request
#import json
import asyncio
import zmq
import datetime
import simplejson as json

from quart import websocket
from quart.blueprints import Blueprint
from functools import wraps
from configs import config as app_config

from qc_compute import MachineQCThread
from machineqc_events import MachineQCEvents
from pynovatec.events import EventRegisterListener,EventListener
from queue import Queue as pyQueue
from threading import Lock

# The Calibration command imports needs to be modified but we will see if this works for now.

# from main import run_calibration
machineqc_status={
    'state': 'Idle',
    'interval': 60,
    'machines': {}
}

machineqc_history = {}

csvpath="/tmp/log/calibration/csv/"
profilepath="/tmp/log/calibration/profile/"
machineqc_config = {
  "CONFIG_FILE_PATH": "/tmp/qc/conf/r_config.json",
  "JSON_DATA_FOLDER_PATH": "/opt/novatec/data/slave/json_data",
  "CONF_FOLDER_PATH": "/tmp/qc/conf",
  "STORED_OBJECTS": "/tmp/qc/stored_objects",
  "stored_objects_dir": "/tmp/qc/stored_objects",
  "ANALYTICS_ERROR_LOG": "/tmp/qc/log/pumpcompute_reason.log",
  "INPUT_JSON_FOLDER_PATH": "/tmp/qc/input_json",
  "OUTPUT_JSON_FOLDER_PATH": "/tmp/qc/output_json",
  "bridge_data_path": "/tmp/qc/ramcsv",
  "machine_filepath": "/opt/novatec/data/slave/json_data",
  "send_input_json": True,
  "working_dir": "/tmp/qc",
  "conf_dir": "/tmp/qc/conf",
  "write_qc_input_json": True,
  "write_qc_output_json": True
}


machineqc_bp = Blueprint('machineqc_controller', __name__)
connected_realtime_data_websockets = list()
machineqc=None
machineqc_mutex = Lock()
def set_asyncio_loop(loop):
    global asyncio_loop
    asyncio_loop = loop
    
async def push_message_out(msg):
    global connected_realtime_data_websockets

    for queue in connected_realtime_data_websockets:
        # await queue.put(msg)
        await queue.put_nowait(msg)

def machineqc_init_event(machines):
    for machine in machines:
        print("machine: ",machine)
        machineqc_status['machines'][machine['machine_id']] = {}
        machineqc_status['machines'][machine['machine_id']]['completed'] = False
        machineqc_status['machines'][machine['machine_id']]['gauges'] = {}
        machineqc_status['machines'][machine['machine_id']]['name'] = machine['name']

def add_qc_history(data):
    global machineqc_history

    if 'machine_id' in data:
        mid = data['machine_id']
        if mid not in machineqc_history:
            machineqc_history[mid] = []
        machineqc_history[mid].append(data)

class MachineQCListener(EventListener):
    def __init__(self,*args,**kwargs):
        #super(EventListener,self).__init__(*args,**kwargs)
        super().__init__(*args,**kwargs)
        self.events.append(MachineQCEvents)

    def handle(self,msg): 
        global asyncio_loop
        asyncio.run_coroutine_threadsafe(push_message_out(json.dumps(msg.data)),asyncio_loop)
        if msg.data['event_name'] == 'machineqc_init':
            machineqc_init_event(msg.data['data'])
        elif msg.data['event_name'] == 'batch_collection_complete':
            machineqc_status['machines'][msg.data['data']['machine_id']]['gauges'] = msg.data['data']
            add_qc_history(msg.data['data'])
        elif msg.data['event_name'] == 'machine_qc_complete':
            machineqc_status['machines'][msg.data['data']['machine_id']]['completed'] = True
        elif msg.data['event_name'] == 'machineqc_shutdown':
            stop_machineqc()

# async def MachineQCWebsocketBroadcast(ws,queue):
async def MachineQCWebsocketBroadcast(queue):
    ws = websocket
    processQueue = True
    while processQueue:
        print("Machineqc awaiting message...")
        message = await queue.get()
        if (message == 'STOP'):
            processQueue = False
            continue
        await ws.send(message)

EventRegisterListener(MachineQCListener())

from quart import copy_current_websocket_context

@machineqc_bp.websocket('/machineqc/')
async def machineqc_data_websocket():
    global connected_realtime_data_websockets
    print("Waiting for data..")
    try:
        queue = Queue()
        connected_realtime_data_websockets.append(queue)
        try:
            print("Machine qc User connected...")
            # task = asyncio.ensure_future(MachineQCWebsocketBroadcast(websocket._get_current_object(),queue))
            # receiver = asyncio.ensure_future(receiving(websocket._get_current_object(),queue))
            task = asyncio.ensure_future(copy_current_websocket_context(MachineQCWebsocketBroadcast)(queue))
            receiver = asyncio.ensure_future(copy_current_websocket_context(receiving)(queue))

            return await receiver
        except Exception as e:
            print("Have ws exception..",str(e))
        finally:
            # websocket disconnected remove the queue from list
            queue.put_nowait("STOP")
            user_disconnect()
            receiver.cancel()
            task.cancel()
            connected_realtime_data_websockets.remove(queue)
            # await task
            #logger.debug('Websocket disconnected for client')
    except Exception as e:
        print("collect_calibrations_data_websocket: ",str(e))
        # close connection, user is not authorized
        await websocket.close(1000, 'UNAUTHORIZED')
        #logger.debug('Websocket disconnected, unauthorized client')
        #raise e

def user_disconnect():
    print("********** user disconnected: ",len(connected_realtime_data_websockets))
    if len(connected_realtime_data_websockets) == 0:
        print("machine qc ------- No clients...")
        #stop_machineqc()


def stop_machineqc():
    global machineqc 
    global machineqc_mutex

    print("***************** MACHINE STOP CALLED **********************")
    machineqc_mutex.acquire()
    if not machineqc:
        print("----- no machineqc")
        machineqc_mutex.release()
        return
    print("stopping machineqc...")
    machineqc.stop()
    machineqc.join()
    machineqc_status['state'] = "Idle"
    print("machineqc is stopped...")
    machineqc= None
    machineqc_mutex.release()

def InitStatus(interval):
    global machineqc_status
    global machineqc_history

    machineqc_status={                                                        
        'state': 'Running',                                                      
        'interval': interval,
        'machines': {}                                                        
    }                                                                           
    machineqc_history = {}


def start_machineqc(params):
    global machineqc
    global machineqc_config
    interval=params['interval']
    machines=params['machines']

    #other arguments weren't being used
    #type=None,mac=None,contents=None,interval=60

    print("start machineqc...")
    machineqc_mutex.acquire()
    if machineqc:
        machineqc_mutex.release()
        raise Exception("Already collecting, must stop before starting..")
    InitStatus(interval)
    machineqc=MachineQCThread(machineqc_config,interval=interval,machines=machines)
    machineqc.start()
    machineqc_mutex.release()

async def send_websocket_response(ws,data,command=None):
    r = {
        "event_name": "response",
        "command": command,
        "data": data
    }
    await ws.send(json.dumps(r))

async def respond_success(ws,command,msg=None):
   await send_websocket_response(ws,{"success": True},command=command)

async def handle_command(ws,jdata):
    global machineqc_status

    # print("Handle command: ",jdata)
    if "cmd" not in jdata:
        raise Exception("No command")
    try:
        if jdata['cmd'] == 'start':
            start_machineqc(params=jdata['params'])
        elif jdata['cmd'] == 'stop':
            stop_machineqc()
        elif jdata['cmd'] == 'status':
            await send_websocket_response(ws,machineqc_status,command=jdata['cmd'])
            return
        elif jdata['cmd'] == 'history':
            await send_websocket_response(ws,machineqc_history,command=jdata['cmd'])
            return
        else:
            raise Exception(f"Invalid command: {jdata['cmd']}")
    except Exception as e:
        await send_websocket_response(ws,{ "error": str(e) },command=jdata['cmd'])
        return
    await respond_success(ws,jdata['cmd'])

# async def receiving(ws,queue):
async def receiving(queue):

    ws = websocket
    while True:
        data = await ws.receive()
        # print("RECV2: ",data)
        try:
            jdata = json.loads(data)
        except:
            #await websocket.send('{"error": "Invalid Json"}')
            await send_websocket_response(ws,{"error": "Invalid Json"})
        # print("Json2: ",jdata)
        try:
            await handle_command(ws,jdata)
        except Exception as e:
            s = str(e)
            await send_websocket_response(ws,{ "error": str(e) })
 
