from quart.blueprints import Blueprint
from configs import config as app_config
import json
from ZmqEventPoster import ZmqEventPoster,ZmqDataEvents
from pynovatec.events import EventListener, EventRegisterListener
from ZmqPostController import user_count_inc, user_count_dec
from quart import websocket
from asyncio import Queue
import asyncio
from queue import Queue as pyQueue

rc_realtime_data_bp = Blueprint('rc_realtime_data_controller', __name__)
connected_realtime_data_websockets = list()
message_queue = []

def push_message_out(msg):
    global message_queue
    for queue in message_queue:
        queue.put(msg)

class ZMQData():
    def __init__(self):        
        self.last_vac = 0
        self.last_ps = 0

    def setLastVac(self, val):
        self.last_vac = val

    def setLastPs(self, val):
        self.last_ps = val

    def process_data(self, val):
        try:
            data = val['data']
            if data['dataType'] == 'vac':
                self.setLastVac(float(data['values'][0]))
            if data['dataType'] == 'ps':
                self.setLastPs(float(data['values'][0]))
            send_data = json.dumps(self.transformData(self.last_vac,self.last_ps))
            # self.broadcastData(send_data)
            push_message_out(send_data)
        except Exception as e:
             print("realtime_data_zmq exception: ",str(e))

    def transformData(self, vac, ps):
        rt = [{
            "vac":    {
                "numbers": {
                    'thresholds': {
                        'min': 0,
                        'max': 16
                    },
                    'marks': [
                        {
                            'mark': 4,
                            'color': 'grey'
                        },
                        {
                            'mark': 11,
                            'color': 'green'
                        },
                        {
                            "mark": 12,
                            "color": "yellow",
                            "alarm": True
                        },
                        {
                            "mark": 16,
                            "color": "red",
                            "alarm": True
                        }
                    ]
                },
                "chart":   {
                    "value": vac,
                    "color": "green",
                    "displayname": "Unloading Railcar",
                    "subassemblyname": "Vacsense",
                    "units": "Hgs"
                }
            },
            "ps":    {
                "numbers": {
                    'thresholds': {
                        'min': 0,
                        'max': 8
                    },
                    'marks': [
                        {
                            'mark': 2,
                            'color': 'grey'
                        },
                        {
                           'mark': 6,
                            'color': 'green'
                        },
                        {
                            "mark": 7,
                            "color": "yellow",
                            "alarm": True
                        },
                        {
                            "mark": 8,
                            "color": "red",
                            "alarm": True
                        }
                    ]
                },
                "chart":   {
                    "value": ps,
                    "color": "green",
                    "displayname": "Filling silo",
                    "subassemblyname": "Pressuresense",
                    "units": "PSI"
                }
            }
        }]
        return rt

async def myWebsocketBroadcast(ws,queue):
    # print("myWebsocketBroadcast..")
    for message in iter(queue.get,'STOP'):
        # print("Have mesage to send: ", message)
        # print("Qsize: ",queue.qsize())
        await ws.send(message)

zmq_data = ZMQData()

class ZmqListener(EventListener):        
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)      
        self.events.append(ZmqDataEvents)  
                                              
    def handle(self,msg):               
        #print("ZMQ Listener run: ",msg.data)
        zmq_data.process_data(msg.data)
        pass

EventRegisterListener(ZmqListener())

# zmq_controller = ZmqController(None,port=app_config['data_stream']['zmq_port'],zmq_data=ZMQData(),host=app_config['data_stream']['zmq_host'])
# ws = WebsocketController(None,zmq_controller=zmq_controller)

@rc_realtime_data_bp.websocket('/railcar/socket/')
async def realtime_data_ws():
    global connected_realtime_data_websockets
    global message_queue

    try:
        queue = Queue()
        mq = pyQueue()
        connected_realtime_data_websockets.append(queue)
        message_queue.append(mq)
        try:
            task = asyncio.ensure_future(myWebsocketBroadcast(websocket._get_current_object(),mq))
            user_count_inc()
            return await connect_websocket(queue)
        finally:
            # websocket disconnected remove the queue from list
            #print('websocket disconnected')
            connected_realtime_data_websockets.remove(queue)
            message_queue.remove(mq)
            user_count_dec()
            task.cancel()
            await task
            
    except Exception as e:
        print("collect_realtime_data_websocket: ",str(e))
        # close connection, user is not authorized
        await websocket.close(1000, 'UNAUTHORIZED')
        user_count_dec()

async def connect_websocket(queue):
    """Called when a new websocket connects"""
    #print('realtime_data_ws: connected')
    while True:
      data = await queue.get()
      try:
        await websocket.send(data)
      except Exception as e:
        pass