import sys

sys.path.append('/opt/novatec/sensor-qc/scripts/')
sys.path.append('/opt/novatec/sensor-qc')

import time
import base64
from asyncio import Queue
import urllib.request
import json
import asyncio
import zmq
import datetime
import simplejson as json

from ZmqCollection import ZmqCollection

from quart import websocket
from quart.blueprints import Blueprint
from functools import wraps
from configs import config as app_config

from VacSensorCollector import VacSensorCollector,set_csv_path,set_profile_path
from PFSensorCollector import PFSensorCollector

from qc_events import QCEvents
from pynovatec.events import EventRegisterListener,EventListener

# The Calibration command imports needs to be modified but we will see if this works for now.

from main import run_calibration

message_queue = []
csvpath="/tmp/log/calibration/csv/"
profilepath="/tmp/log/calibration/profile/"

set_csv_path(csvpath)
set_profile_path(profilepath)

class CalibrationListener(EventListener):
    def __init__(self,*args,**kwargs):
        #super(EventListener,self).__init__(*args,**kwargs)
        super().__init__(*args,**kwargs)
        self.events.append(QCEvents)

    def handle(self,msg):
        global asyncio_loop                  
                                         
        asyncio.run_coroutine_threadsafe(push_message_out(json.dumps(msg.data)),asyncio_loop)
        if msg.data['event_name'] == 'collection_complete':
            #stop_zmq()
            profile_file = msg.data['data']['profile_file']
            result = run_calibration(profile_file)
            result['sensor_id'] = msg.data['data'].get('sensor_id')
            print("Calibration Result: ",result)
            asyncio.run_coroutine_threadsafe(push_message_out(json.dumps({
                "event_name": "calibration_result",
                "data": result
            })), asyncio_loop)
        if msg.data['event_name'] == 'calibration_complete':
            stop_zmq()

EventRegisterListener(CalibrationListener())

calibration_bp = Blueprint('calibration_controller', __name__)
connected_realtime_data_websockets = list()
zmq_connect=None

def set_asyncio_loop(loop):
    global asyncio_loop
    asyncio_loop = loop
    
async def push_message_out(msg):
    global connected_realtime_data_websockets
    for queue in connected_realtime_data_websockets:
        await queue.put(msg)

async def myWebsocketBroadcast(queue):
    # print("myWebsocketBroadcast..")
    processQueue = True
    while processQueue:
        message = await queue.get()
        if (message == 'STOP'):
            processQueue = False
        await websocket.send(message)

@calibration_bp.websocket('/calibration/')
async def collect_calibrations_data_websocket():
    global connected_realtime_data_websockets
    print("Waiting for data..")
    try:
        queue = Queue()
        connected_realtime_data_websockets.append(queue)
        print('Websocket connected for client')
        try:
            task = asyncio.ensure_future(myWebsocketBroadcast(queue))
            receiver = asyncio.ensure_future(receiving(queue))
            return await receiver
        except Exception as e:
            print("Have ws exception..",str(e))
        finally:
            # websocket disconnected remove the queue from list
            print("User disconnected...")
            connected_realtime_data_websockets.remove(queue)
            user_disconnect()
            task.cancel()
            await task
            #logger.debug('Websocket disconnected for client')
    except Exception as e:
        print("collect_calibrations_data_websocket: ",str(e))
        # close connection, user is not authorized
        await websocket.close(1000, 'UNAUTHORIZED')
        #logger.debug('Websocket disconnected, unauthorized client')
        #raise e


def user_disconnect():
    print("********** user disconnected: ",len(connected_realtime_data_websockets))
    if len(connected_realtime_data_websockets) == 0:
        print("------- No clients...")
        stop_zmq()


def getDataCollector(package_type,mac,package_index=1,contents=None):
    m = mac.lower()
    print("m: ",m)
    if package_type.upper() == "VAC":
        return VacSensorCollector(package_type,m,package_index,contents=contents)
    if package_type.upper() == "PF":
        return PFSensorCollector(package_type,m,package_index,contents=contents)
    return None


def get_collectors(sensors):
    collectors = []
    for sensor in sensors:
        contents=None
        if 'on' in sensor:
            contents = {
                'on': sensor.get('on'),
                'off': sensor.get('off')
            }
        collector = getDataCollector(sensor['type'],sensor['mac'],sensor['index'],contents=contents)
        collectors.append(collector)
    return collectors

def stop_zmq():
    global zmq_connect

    print("***************** ZMQ STOP CALLED **********************")
    if not zmq_connect:
        print("----- no zmq_connect")
        return
    print("stopping zmq...")
    zmq_connect.stop()
    zmq_connect.join()
    print("Zmq is stopped...")
    zmq_connect = None

def start_zmq(params=None,type=None,mac=None,contents=None):
    global zmq_connect

    if zmq_connect:
        raise Exception("Already collecting, must stop before starting..")
    if params is None:
        raise Exception("No parameters give.")

    collector = None
    if 'sensors' in params:
        collectors = get_collectors(params['sensors'])
    else:
        type = params['type']
        mac = params['mac']
        print("type: ",type)
        collector=getDataCollector(type,mac,contents=contents)
        collectors = [collector]

    if not collectors:
        #raise Exception(f"Could not create collector for type: {type}")
        return
    #collector=VacSensorCollector("VAC","9824968e08a9",1)
    #zmq_connect=ZmqCollection(collector,port=7778,contents=contents)
    zmq_connect=ZmqCollection(None,port=7778,collectors=collectors)
    zmq_connect.start()


async def send_websocket_response(data,command=None):
    r = {
        "event_name": "response",
        "command": command,
        "data": data
    }
    await websocket.send(json.dumps(r))


async def respond_success(command,msg=None):
   await send_websocket_response({"success": True},command=command)

async def sending(queue):
    while True:
        print("Awaiting data...")
        data = await queue.get()
        print("Have data sending it..")
        await websocket.send(data)
    print("Sending ended...")


async def handle_command(jdata):
    print("handle command..")
    if "cmd" not in jdata:
        raise Exception("No command")
    try:
        if jdata['cmd'] == 'start':
            print("start command")
            if 'params' in jdata:
                contents=None
                if 'off' in jdata['params']:
                    contents = {
                        'off': float(jdata['params']['off']), 
                        'on': float(jdata['params']['on']) 
                    }
            #start_zmq(type=jdata['params']['type'],mac=jdata['params']['mac'],contents=contents)
            start_zmq(params=jdata['params'],contents=contents)
        elif jdata['cmd'] == 'stop':
            print("stop command")
            stop_zmq()
        else:
            raise Exception(f"Invalid command: {jdata['cmd']}")
    except Exception as e:
        await send_websocket_response({ "error": str(e) },command=jdata['cmd'])
        return
    await respond_success(jdata['cmd'])

async def receiving(queue):
    while True:
        data = await websocket.receive()
        print("RECV: ",data)
        try:
            jdata = json.loads(data)
            print("Json: ",jdata)
            try:
                await handle_command(jdata)
            except Exception as e:
                s = str(e)
                await send_websocket_response({ "error": str(e) })
        except:
            #await websocket.send('{"error": "Invalid Json"}')
            await send_websocket_response({"error": "Invalid Json"})
 

# @calibration_bp.websocket('/calibration/')
# @collect_calibrations_data_websocket
# async def realtime_data_ws(queue):
#     """Called when a new websocket connects"""
#     print("User connected...")
#     sender = asyncio.create_task(sending(queue))
#     receiver = asyncio.create_task(receiving(queue))

#     await asyncio.gather(sender,receiver)
#     print("Gather finished...")


# def broadcast_sensor_data(dataItem):
#     global asyncio_loop;

#     if dataItem:
#         for queue in connected_realtime_data_websockets:
#             asyncio.run_coroutine_threadsafe(queue.put(dataItem),asyncio_loop)



	
